#include <math.h>
#include <imsls.h>
#include <stdio.h>

void l_print_LSD(int n1, int* equalMeans, float *means);

int main()
{
    char **anova_row_labels = NULL;
    char **loc_row_labels   = NULL;
    char *col_labels[] = {" ", "\nID", "\nDF", "\nSSQ  ", 
        "Mean  \nsquares", "\nF-Test", "\np-Value"};
    float alpha = 0.05;
    int i, l, page_width = 132;

    int n = 100;           /* Total number of observations      */
    int n_locations  = 2;  /* Number of locations               */
    int n_treatments =25;  /* Number of treatments              */
    int n_reps       = 2;  /* Number of replicates/location     */
    int n_blocks     =10;  /* Total number of blocks/location   */
    int n_aov_rows   = 7;  /* Number of rows in the anova table */

    int rep[]={
        1, 1, 1, 1, 1,
        1, 1, 1, 1, 1,
        1, 1, 1, 1, 1,
        1, 1, 1, 1, 1,
        1, 1, 1, 1, 1,
        2, 2, 2, 2, 2,
        2, 2, 2, 2, 2,
        2, 2, 2, 2, 2,
        2, 2, 2, 2, 2,
        2, 2, 2, 2, 2,
        1, 1, 1, 1, 1,
        1, 1, 1, 1, 1,
        1, 1, 1, 1, 1,
        1, 1, 1, 1, 1,
        1, 1, 1, 1, 1,
        2, 2, 2, 2, 2,
        2, 2, 2, 2, 2,
        2, 2, 2, 2, 2,
        2, 2, 2, 2, 2,
        2, 2, 2, 2, 2
    };

    int block[]={        
        1,  1,  1,  1,  1,
        2,  2,  2,  2,  2,
        3,  3,  3,  3,  3,
        4,  4,  4,  4,  4,
        5,  5,  5,  5,  5,
        6,  6,  6,  6,  6, 
        7,  7,  7,  7,  7,
        8,  8,  8,  8,  8, 
        9,  9,  9,  9,  9,
        10, 10, 10, 10, 10,
        1,  1,  1,  1,  1,
        2,  2,  2,  2,  2,
        3,  3,  3,  3,  3,
        4,  4,  4,  4,  4,
        5,  5,  5,  5,  5,
        6,  6,  6,  6,  6, 
        7,  7,  7,  7,  7,
        8,  8,  8,  8,  8, 
        9,  9,  9,  9,  9,
        10, 10, 10, 10, 10
    };

    int treatment[]={ 
        1,  2,  3,  4,  5,
        6,  7,  8,  9, 10,
        11, 12, 13, 14, 15,
        16, 17, 18, 19, 20,
        21, 22, 23, 24, 25,
        1,  6, 11, 16, 21,
        2,  7, 12, 17, 22,
        3,  8, 13, 18, 23,
        4,  9, 14, 19, 24,
        5, 10, 15, 20, 25,
        1,  2,  3,  4,  5,
        6,  7,  8,  9, 10,
        11, 12, 13, 14, 15,
        16, 17, 18, 19, 20,
        21, 22, 23, 24, 25,
        1,  6, 11, 16, 21,
        2,  7, 12, 17, 22,
        3,  8, 13, 18, 23,
        4,  9, 14, 19, 24,
        5, 10, 15, 20, 25
    };
    int location[]={
        1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
        1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
        1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
        1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
        1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
        2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
        2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
        2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
        2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
        2, 2, 2, 2, 2, 2, 2, 2, 2, 2
    };

    float y[] ={
        6,  7,  5,  8,  6,
        16, 12, 12, 13,  8,
        17,  7,  7,  9, 14,
        18, 16, 13, 13, 14,
        14, 15, 11, 14, 14,
        24, 13, 24, 11,  8,
        21, 11, 14, 11, 23,
        16,  4, 12, 12, 12,
        17, 10, 30,  9, 23,
        15, 15, 22, 16, 19,
        13, 26,  9, 13, 11,
        15, 18, 22, 11, 15,
        19, 10, 10, 10, 16,
        21, 16, 17,  4, 17,
        15, 12, 13, 20,  8,
        16,  7, 20, 13, 21,
        15, 10, 11,  7, 14,
        7, 11, 15, 15, 16,
        19, 14, 20,  6, 16,
        17, 18, 20, 15, 14
    };

    float grand_mean;
    float cv;
    float *aov;
    float *location_anova_table;
    float *loc_anova_table;
    float *treatment_means;
    float *std_err;
    int   df;
    int   n_missing;
    int   *equal_means;

    /* Set first observation to missing. */
    y[0] = imsls_f_machine(6);

    aov = imsls_f_lattice(n, n_locations, n_reps, n_blocks,  
        n_treatments, rep, block, treatment, y, 
        IMSLS_LOCATIONS, location,
        IMSLS_GRAND_MEAN, &grand_mean, 
        IMSLS_CV, &cv,
        IMSLS_TREATMENT_MEANS, &treatment_means, 
        IMSLS_STD_ERRORS, &std_err,
        IMSLS_LOCATION_ANOVA_TABLE, &location_anova_table,
        IMSLS_ANOVA_ROW_LABELS, &anova_row_labels,
        IMSLS_N_MISSING, &n_missing,
        0);

    /* Output results. */

    imsls_page(IMSLS_SET_PAGE_WIDTH, &page_width);
    /* Print the ANOVA table. */
    imsls_f_write_matrix("   *** ANALYSIS OF VARIANCE TABLE ***", 
        7, 6, aov, 
        IMSLS_WRITE_FORMAT, "%3.0f%3.0f%8.2f%7.2f%7.2f%9.3g",
        IMSLS_ROW_LABELS, anova_row_labels,
        IMSLS_COL_LABELS, col_labels,
        0);

    /* Print the location ANOVA tables. */
    for (i=0; i < n_locations; i++){
        printf("\n\n\t\t\t\tLOCATION %d", i+1);
        imsls_f_write_matrix("   *** ANALYSIS OF VARIANCE TABLE ***", 
            7, 6, &(location_anova_table[i*42]), 
            IMSLS_WRITE_FORMAT, "%3.0f%3.0f%8.2f%7.2f%7.2f%9.3g",
            IMSLS_ROW_LABELS, anova_row_labels,
            IMSLS_COL_LABELS, col_labels,
            0);
    }

    printf("\n\nAdjusted Grand Mean:      %7.3f", grand_mean);
    printf("\n\nCoefficient of Variation: %7.3f\n\n", cv);
    l = 0;
    printf("Adjusted Treatment Means: \n");
    for (i=0; i < n_treatments; i++){
        printf("treatment[%2d]              %7.4f \n", i+1,
            treatment_means[l++]);
    }
    df = (int) std_err[3];
    printf("\nStandard Error for Comparing Two Adjusted Treatment ");
    printf("Means: %f \n(df=%d)\n", std_err[2], df);
    equal_means = imsls_f_multiple_comparisons(n_treatments,
        treatment_means, df, std_err[2]/sqrt(2), IMSLS_LSD,
        IMSLS_ALPHA, alpha,0);
    l_print_LSD(n_treatments, equal_means, treatment_means);

    printf("\n\nNumber of missing observations: %d\n", n_missing);

}
/*
* Function to display means comparison.
*/
void l_print_LSD(int n, int *equalMeans, float *means){
    float x=0.0;
    int i, j, k;
    int iSwitch;
    int *idx;

    idx = (int *) malloc(n * sizeof (int));

    for (k=0; k < n; k++) {
        idx[k]   =k+1;
    }

    /* Sort means in ascending order*/

    iSwitch=1;
    while (iSwitch != 0){
        iSwitch = 0;
        for (i = 0; i < n-1; i++){
            if (means[i] > means[i+1]){
                iSwitch = 1;
                x = means[i];
                means[i] = means[i+1];
                means[i+1] = x;
                j = idx[i];
                idx[i] = idx[i+1];
                idx[i+1] = j;
            }
        }
    }
    printf("[group] \t  Mean \t\tLSD Grouping \n");
    for (i=0; i < n; i++){
        printf("  [%d] \t\t%f", idx[i], means[i]);                
        for (j=1; j < i+1; j++){
            if(equalMeans[j-1] >= i+2-j){
                printf("\t  *");
            }else{ 
                if(equalMeans[j-1]>0) printf("\t");
            }
        }
        if (i < n-1 && equalMeans[i]>0) printf("\t  *");
        printf("\n");
    }
    imsls_free(idx);
    idx = NULL;
    return;
}
